#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Enter the description of your new custom indicator here
    /// </summary>
    [Description("Enter the description of your new custom indicator here")]
    [Gui.Design.DisplayName("Ergodic")]
    public class Ergodic : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int fast = 3; // Default setting for Fast
            private int slow = 14; // Default setting for Slow
            private int signalLen = 5; // Default setting for SignalLen
		private DataSeries signalLine;
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.DarkGreen), PlotStyle.Line, "SignalAboveZero"));
				Plots[0].Pen.Width = 3;
			Add(new Plot(Color.FromKnownColor(KnownColor.LimeGreen), PlotStyle.Line, "SignalUp"));
				Plots[1].Pen.Width = 3;
			Add(new Plot(Color.FromKnownColor(KnownColor.OrangeRed), PlotStyle.Line, "SignalDown"));
				Plots[2].Pen.Width = 3;
			Add(new Plot(Color.FromKnownColor(KnownColor.Black), PlotStyle.Line, "SignalNeutral"));
				Plots[3].Pen.Width = 3;
			Add(new Plot(Color.FromKnownColor(KnownColor.Firebrick), PlotStyle.Line, "SignalBelowZero"));
				Plots[4].Pen.Width = 3;
            Add(new Plot(Color.FromKnownColor(KnownColor.Black), PlotStyle.Line, "Main"));
				Plots[5].Pen = new Pen(Color.Black, 2);
            Add(new Plot(Color.FromKnownColor(KnownColor.Lime), PlotStyle.Bar, "MainUp"));
				Plots[6].Pen = new Pen(Color.Lime, 4);
            Add(new Plot(Color.FromKnownColor(KnownColor.Magenta), PlotStyle.Bar, "MainDown"));
				Plots[7].Pen = new Pen(Color.Magenta, 4);
			Add(new Plot(Color.Red, PlotStyle.Dot,"CrossDown"));
				Plots[8].Pen = new Pen(Color.Red, 4);
			Add(new Plot(Color.Lime, PlotStyle.Dot,"CrossUp"));
				Plots[9].Pen = new Pen(Color.Green, 4);			
            Add(new Line(Color.FromKnownColor(KnownColor.DarkGray), 0, "Zero"));
			            Add(new Line(Color.FromKnownColor(KnownColor.DarkGray), 25, "UpperLevel"));
			            Add(new Line(Color.FromKnownColor(KnownColor.DarkGray), -25, "LowerLevel"));
            CalculateOnBarClose	= false;
            Overlay				= false;
            PriceTypeSupported	= false;
			
			signalLine   = new DataSeries(this);
			
			// Thresholds for changing colors.  Plots[6] is the '7th' plot = MainUp 
			
			Plots[6].Min = 0;  
			Plots[7].Max = 0; 			
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
if (CurrentBar < 25)
			{
				
				return	;		
			}
			else
			{
				double ergo 		= TSI(Close,fast,slow)[0] ;
				double prev_ergo 	= TSI(Close,fast,slow)[1] ;
				Main.Set(ergo);						// Plot it
				MainUp.Set(ergo);						// Plot it
				MainDown.Set(ergo);						// Plot it
				double signal 		= EMA(TSI(Close,fast,slow),signalLen)[0];
				double prev_signal 	= EMA(TSI(Close,fast,slow),signalLen)[1];				
				//Signal.Set(signal);
				signalLine.Set(signal);
				
				// Draw a dot where the signal and main lines cross.  Set a plot to +/- 1 to indicate long/short
				if(signal>ergo && prev_signal <= prev_ergo && ergo > 20) 
				{
					Values[8].Set(signal);
				}
				if(signal<ergo && prev_signal >= prev_ergo && ergo < -20) 
				{
					Values[9].Set(signal);
				}
				
				//Signal line color change
				
				if (Rising(signalLine) && signal > 0)
			{
				// Connects the rising plot segment with the other plots
				SignalAboveZero.Set(1, signalLine[1]);
				
				// Adds the new rising plot line segment to the line
				SignalAboveZero.Set(signalLine[0]);
			}
				
			else if (Rising(signalLine) && signal <= 0)
			{
				// Connects the rising plot segment with the other plots
				SignalUp.Set(1, signalLine[1]);
				
				// Adds the new rising plot line segment to the line
				SignalUp.Set(signalLine[0]);
			}
			
			else if (Falling(signalLine) && signal >= 0)
			{
				// Connects the new falling plot segment with the rest of the line
				SignalDown.Set(1, signalLine[1]);
				
				// Adds the new falling plot line segment to the line
				SignalDown.Set(signalLine[0]);
			}
			
			else if (Falling(signalLine) && signal < 0)
			{
				// Connects the neutral plot segment with the rest of the line
				SignalBelowZero.Set(1, signalLine[1]);
				
				// Adds the new neutral plot line segment to the line
				SignalBelowZero.Set(signalLine[0]);
            }
			
			else
			{
				// Connects the neutral plot segment with the rest of the line
				SignalNeutral.Set(1, signalLine[1]);
				
				// Adds the new neutral plot line segment to the line
				SignalNeutral.Set(signalLine[0]);
            }
			}
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SignalAboveZero
        {
            get { return Values[0]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SignalUp
        {
            get { return Values[1]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SignalDown
        {
            get { return Values[2]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SignalNeutral
        {
            get { return Values[3]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SignalBelowZero
        {
            get { return Values[4]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Main
        {
            get { return Values[5]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MainUp
        {
            get { return Values[6]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MainDown
        {
            get { return Values[7]; }
        }
		

        [Description("")]
		[Gui.Design.DisplayName ("\t\tFast Length?")]
        [Category("Parameters")]
        public int Fast
        {
            get { return fast; }
            set { fast = Math.Max(1, value); }
        }

        [Description("")]
		[Gui.Design.DisplayName ("\tSlow Length?")]
        [Category("Parameters")]
        public int Slow
        {
            get { return slow; }
            set { slow = Math.Max(1, value); }
        }

        [Description("EMA for Signal Line")]
		[Gui.Design.DisplayName ("Signal Line?")]
        [Category("Parameters")]
        public int SignalLen
        {
            get { return signalLen; }
            set { signalLen = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private Ergodic[] cacheErgodic = null;

        private static Ergodic checkErgodic = new Ergodic();

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Ergodic Ergodic(int fast, int signalLen, int slow)
        {
            return Ergodic(Input, fast, signalLen, slow);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Ergodic Ergodic(Data.IDataSeries input, int fast, int signalLen, int slow)
        {
            if (cacheErgodic != null)
                for (int idx = 0; idx < cacheErgodic.Length; idx++)
                    if (cacheErgodic[idx].Fast == fast && cacheErgodic[idx].SignalLen == signalLen && cacheErgodic[idx].Slow == slow && cacheErgodic[idx].EqualsInput(input))
                        return cacheErgodic[idx];

            lock (checkErgodic)
            {
                checkErgodic.Fast = fast;
                fast = checkErgodic.Fast;
                checkErgodic.SignalLen = signalLen;
                signalLen = checkErgodic.SignalLen;
                checkErgodic.Slow = slow;
                slow = checkErgodic.Slow;

                if (cacheErgodic != null)
                    for (int idx = 0; idx < cacheErgodic.Length; idx++)
                        if (cacheErgodic[idx].Fast == fast && cacheErgodic[idx].SignalLen == signalLen && cacheErgodic[idx].Slow == slow && cacheErgodic[idx].EqualsInput(input))
                            return cacheErgodic[idx];

                Ergodic indicator = new Ergodic();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Fast = fast;
                indicator.SignalLen = signalLen;
                indicator.Slow = slow;
                Indicators.Add(indicator);
                indicator.SetUp();

                Ergodic[] tmp = new Ergodic[cacheErgodic == null ? 1 : cacheErgodic.Length + 1];
                if (cacheErgodic != null)
                    cacheErgodic.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheErgodic = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Ergodic Ergodic(int fast, int signalLen, int slow)
        {
            return _indicator.Ergodic(Input, fast, signalLen, slow);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.Ergodic Ergodic(Data.IDataSeries input, int fast, int signalLen, int slow)
        {
            return _indicator.Ergodic(input, fast, signalLen, slow);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Ergodic Ergodic(int fast, int signalLen, int slow)
        {
            return _indicator.Ergodic(Input, fast, signalLen, slow);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.Ergodic Ergodic(Data.IDataSeries input, int fast, int signalLen, int slow)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.Ergodic(input, fast, signalLen, slow);
        }
    }
}
#endregion
